class DeadIssue(Exception):
    pass


class ExistsError(Exception):
    pass


class UnknownIdentityError(Exception):
    pass


class VerifyError(Exception):
    pass


class SignError(Exception):
    pass
